set phi1 ""
set phi2 ""
set phz1 ""
set phz2 ""
set eta ""
set cut ""
set kdp ""
set g 9.81
set xki 120.0
set xkf 165.0

proc TdynTcl_StartNewFluidStep {} {
    global phi1 phi2 phz1 phz2 eta cut kdp g xki xkf

    set dt [TdynTcl_Dt]
    set dtg [expr 12.0/$dt/$dt/$g]
    set nnod [TdynTcl_NNode 1]

    if { $phi1 eq "" } { set phi1 [::mather::mkvector $nnod 0.0] }
    if { $phi2 eq "" } { set phi2 [vmexpr temp=fph1] }
    if { $phz1 eq "" } { set phz1 [::mather::mkvector $nnod 0.0] }
    if { $phz2 eq "" } { set phz2 [vmexpr temp=-$dtg*fph1] } 
    if { $eta eq "" } { set eta [::mather::mkvector $nnod 0.0] }
    if { $kdp eq "" } { 
        set kdp [::mather::mkvector $nnod 0.0]
        set nodelist [TdynTcl_GetFluidBodyNodes frees]
        foreach inode $nodelist {
            set x [expr ([TdynTcl_Coord $inode 1]-$xki)/($xkf-$xki)]
            if { $x>0 & $x<=1 } { 
                ::mather::setelem $kdp $inode [expr 3.0*$x*$x-2.0*$x*$x*$x]
            }
        }
    }
    if { $cut eq "" } { 
        set cut [::mather::mkvector $nnod 0.0]
        set nodelist [TdynTcl_GetFluidBodyNodes frees]
        foreach inode $nodelist {
            ::mather::setelem $cut $inode 1.0
        }
    }
}

proc TdynTcl_AssembleFluidPhiVariable { index } {
    global phi1 phi2 phz1 phz2 eta cut kdp g xki xkf

    if { $index != 1 } { return }

    TdynTcl_Clock TdynTcl_AssembleFluidPhiVariable start

    set nnod [TdynTcl_NNode 1]
    set step [TdynTcl_Step]
    set dt [TdynTcl_Dt]   
    set dtg [expr 1.0/$dt/$dt/$g]

    # Assemble free surface bc
    # Assemble (left hand side)
    set sigm [vmexpr temp=inverse([expr 1.0/12.0]+[expr 1.0/(2.0*$dt)]*$kdp)]
    set temp [vmexpr temp=$dtg*$sigm]
    vmmatrix_vector_mult_add fsys $temp ffrees 3
    # Assemble (right hand side)
    set gamp [vmexpr temp=[expr 1.0/12.0]-[expr 1.0/(2.0*$dt)]*$kdp]
    vmmult_vector_per_vec $gamp $phz1 $gamp
    vmexpr $temp=[expr 2.0*$dtg]*$phi2-$dtg*$phi1-[expr 5./6.]*$phz2-1.0*$gamp
    vmmult_vector_per_vec $temp $sigm $temp    
    vmmult_matrix_per_vec_add ffrees $temp frhs 
    vmdelete $temp
    vmdelete $sigm
    vmdelete $gamp

    TdynTcl_Clock TdynTcl_AssembleFluidPhiVariable end
}

proc TdynTcl_FinishFluidStep {} {
    global phi1 phi2 phz1 phz2 eta cut kdp g xki xkf

    TdynTcl_Clock TdynTcl_FinishFluidStep start

    set dt [TdynTcl_Dt] 
    set dtg [expr 1.0/$dt/$dt/$g]

    # Update phiz values (si no se usa cut, phz2 diverge en los nodos no usados)
    set gamp [vmexpr temp=[expr 1.0/12.0]-[expr 1.0/(2.0*$dt)]*$kdp]
    vmmult_vector_per_vec $gamp $phz1 $gamp
    set temp [vmexpr temp=-$dtg*fph1+[expr 2.0*$dtg]*$phi2-$dtg*$phi1-[expr 5./6.]*$phz2-1.0*$gamp]
    vmexpr $phz1=$phz2
    set sigm [vmexpr temp=inverse([expr 1.0/12.0]+[expr 1.0/(2.0*$dt)]*$kdp)]
    vmmult_vector_per_vec $temp $sigm $temp
    vmmult_vector_per_vec $cut $temp $phz2
    vmdelete $gamp
    vmdelete $sigm
    vmdelete $temp
    # Update phi values
    vmexpr $phi1=$phi2
    vmexpr $phi2=fph1     
    # Update eta values
    set k1 [expr 1.0/$dt/$g]
    set k2 [expr $dt/6]
    vmexpr $eta=-$k1*$phi2+$k1*$phi1+[expr 2.0*$k2]*$phz2+$k2*$phz1
    vmmult_vector_per_vec $eta $cut $eta

    TdynTcl_Clock TdynTcl_FinishFluidStep end
}

proc calc_PhiZ { } {
    global phi1 phi2 phz1 phz2 eta g
    set inode [TdynTcl_Index 1]
    return [::mather::getelem $phz2 $inode]
}

proc calc_Eta { } {
    global phi1 phi2 phz1 phz2 eta g
    set inode [TdynTcl_Index 1]
    return [::mather::getelem $eta $inode]
}